@@ -74,11 +74,11 @@ data MapDescriptor d =
    MapDescriptor (d, d) Stride Order NumComponents
    deriving ( Eq, Ord, Show )
 
-totalComponents1 :: Domain d => MapDescriptor d -> Int
+totalComponents1 :: MapDescriptor d -> Int
 totalComponents1 (MapDescriptor _ stride order numComp) =
    fromIntegral stride * (fromIntegral order - 1) + fromIntegral numComp
 
-totalComponents2 :: Domain d => MapDescriptor d -> MapDescriptor d -> Int
+totalComponents2 :: MapDescriptor d -> MapDescriptor d -> Int
 totalComponents2 uDescriptor vDescriptor@(MapDescriptor _ _ _ numComp) =
    totalComponents1 uDescriptor + totalComponents1 vDescriptor - fromIntegral numComp
 
