@@ -2,7 +2,7 @@
 --------------------------------------------------------------------------------
 -- |
 -- Module      :  Graphics.Rendering.OpenGL.GL.Face
--- Copyright   :  (c) Sven Panne 2002-2013
+-- Copyright   :  (c) Sven Panne 2002-2016
 -- License     :  BSD3
 -- 
 -- Maintainer  :  Sven Panne <svenpanne@gmail.com>
@@ -17,7 +17,7 @@ module Graphics.Rendering.OpenGL.GL.Face (
    Face(..), marshalFace, unmarshalFace
 ) where
 
-import Graphics.Rendering.OpenGL.Raw
+import Graphics.GL
 
 --------------------------------------------------------------------------------
 
@@ -29,13 +29,13 @@ data Face =
 
 marshalFace :: Face -> GLenum
 marshalFace x = case x of
-   Front -> gl_FRONT
-   Back -> gl_BACK
-   FrontAndBack -> gl_FRONT_AND_BACK
+   Front -> GL_FRONT
+   Back -> GL_BACK
+   FrontAndBack -> GL_FRONT_AND_BACK
 
 unmarshalFace :: GLenum -> Face
 unmarshalFace x
-   | x == gl_FRONT = Front
-   | x == gl_BACK = Back
-   | x == gl_FRONT_AND_BACK = FrontAndBack
+   | x == GL_FRONT = Front
+   | x == GL_BACK = Back
+   | x == GL_FRONT_AND_BACK = FrontAndBack
    | otherwise = error ("unmarshalFace: illegal value " ++ show x)
