@@ -2,7 +2,7 @@
 --------------------------------------------------------------------------------
 -- |
 -- Module      :  Graphics.Rendering.OpenGL.GL.GLboolean
--- Copyright   :  (c) Sven Panne 2002-2013
+-- Copyright   :  (c) Sven Panne 2002-2016
 -- License     :  BSD3
 -- 
 -- Maintainer  :  Sven Panne <svenpanne@gmail.com>
@@ -17,14 +17,14 @@ module Graphics.Rendering.OpenGL.GL.GLboolean (
    marshalGLboolean, unmarshalGLboolean
 ) where
 
-import Graphics.Rendering.OpenGL.Raw
+import Graphics.GL
 
 --------------------------------------------------------------------------------
 
 marshalGLboolean :: Num a => Bool -> a
 marshalGLboolean x = fromIntegral $ case x of
-   False -> gl_FALSE
-   True -> gl_TRUE
+   False -> GL_FALSE
+   True -> GL_TRUE
 
 unmarshalGLboolean :: (Eq a, Num a) => a -> Bool
-unmarshalGLboolean = (/= fromIntegral gl_FALSE)
+unmarshalGLboolean = (/= fromIntegral GL_FALSE)
