@@ -2,7 +2,7 @@
 --------------------------------------------------------------------------------
 -- |
 -- Module      :  Graphics.Rendering.OpenGL.GL.Shaders.Shader
--- Copyright   :  (c) Sven Panne 2013
+-- Copyright   :  (c) Sven Panne 2016
 -- License     :  BSD3
 -- 
 -- Maintainer  :  Sven Panne <svenpanne@gmail.com>
@@ -22,7 +22,7 @@ import Data.ObjectName
 import Graphics.Rendering.OpenGL.GL.DebugOutput
 import Graphics.Rendering.OpenGL.GL.GLboolean
 import Graphics.Rendering.OpenGL.GL.QueryUtils
-import Graphics.Rendering.OpenGL.Raw
+import Graphics.GL
 
 --------------------------------------------------------------------------------
 
@@ -34,4 +34,4 @@ instance ObjectName Shader where
    deleteObjectName = liftIO . glDeleteShader . shaderID
 
 instance CanBeLabeled Shader where
-   objectLabel = objectNameLabel gl_SHADER . shaderID
+   objectLabel = objectNameLabel GL_SHADER . shaderID
