@@ -133,6 +133,25 @@ data PixelInternalFormat =
    | StencilIndex4
    | StencilIndex8
    | StencilIndex16
+   | RGBS3TC
+   | RGB4S3TC
+   | RGBAS3TC
+   | RGBA4S3TC
+   | RGBADXT5S3TC
+   | RGBA4DXT5S3TC
+   | CompressedRGBAS3TCDXT1
+   | CompressedRGBAS3TCDXT3
+   | CompressedRGBAS3TCDXT5
+   | CompressedRGBS3TCDXT1
+   | Alpha32F
+   | Intensity32F
+   | Luminance32F
+   | LuminanceAlpha32F
+   | Alpha16F
+   | Intensity16F
+   | Luminance16F
+   | LuminanceAlpha16F
+   | Depth24Stencil8
    deriving ( Eq, Ord, Show )
 
 marshalPixelInternalFormat :: PixelInternalFormat -> GLint
@@ -247,6 +266,25 @@ marshalPixelInternalFormat x = fromIntegral $ case x of
    StencilIndex4 -> GL_STENCIL_INDEX4
    StencilIndex8 -> GL_STENCIL_INDEX8
    StencilIndex16 -> GL_STENCIL_INDEX16
+   RGBS3TC -> GL_RGB_S3TC
+   RGB4S3TC -> GL_RGB4_S3TC
+   RGBAS3TC -> GL_RGBA_S3TC
+   RGBA4S3TC -> GL_RGBA4_S3TC
+   RGBADXT5S3TC -> GL_RGBA_DXT5_S3TC
+   RGBA4DXT5S3TC -> GL_RGBA4_DXT5_S3TC
+   CompressedRGBAS3TCDXT1 -> GL_COMPRESSED_RGBA_S3TC_DXT1_EXT
+   CompressedRGBAS3TCDXT3 -> GL_COMPRESSED_RGBA_S3TC_DXT3_EXT
+   CompressedRGBAS3TCDXT5 -> GL_COMPRESSED_RGBA_S3TC_DXT5_EXT
+   CompressedRGBS3TCDXT1 -> GL_COMPRESSED_RGB_S3TC_DXT1_EXT
+   Alpha32F -> GL_ALPHA32F_ARB
+   Intensity32F -> GL_INTENSITY32F_ARB
+   Luminance32F -> GL_LUMINANCE32F_ARB
+   LuminanceAlpha32F -> GL_LUMINANCE_ALPHA32F_ARB
+   Alpha16F -> GL_ALPHA16F_ARB
+   Intensity16F -> GL_INTENSITY16F_ARB
+   Luminance16F -> GL_LUMINANCE16F_ARB
+   LuminanceAlpha16F -> GL_LUMINANCE_ALPHA16F_ARB
+   Depth24Stencil8 -> GL_DEPTH24_STENCIL8_EXT
 
 -- *sigh* The OpenGL API is sometimes a bit creative in its usage of types...
 marshalPixelInternalFormat' :: PixelInternalFormat -> GLenum
@@ -359,6 +397,25 @@ unmarshalPixelInternalFormat x
    | y == GL_STENCIL_INDEX4 = StencilIndex4
    | y == GL_STENCIL_INDEX8 = StencilIndex8
    | y == GL_STENCIL_INDEX16 = StencilIndex16
+   | y == GL_RGB_S3TC = RGBS3TC
+   | y == GL_RGB4_S3TC = RGB4S3TC
+   | y == GL_RGBA_S3TC = RGBAS3TC
+   | y == GL_RGBA4_S3TC = RGBA4S3TC
+   | y == GL_RGBA_DXT5_S3TC = RGBADXT5S3TC
+   | y == GL_RGBA4_DXT5_S3TC = RGBA4DXT5S3TC
+   | y == GL_COMPRESSED_RGBA_S3TC_DXT1_EXT = CompressedRGBAS3TCDXT1
+   | y == GL_COMPRESSED_RGBA_S3TC_DXT3_EXT = CompressedRGBAS3TCDXT3
+   | y == GL_COMPRESSED_RGBA_S3TC_DXT5_EXT = CompressedRGBAS3TCDXT5
+   | y == GL_COMPRESSED_RGB_S3TC_DXT1_EXT = CompressedRGBS3TCDXT1
+   | y == GL_ALPHA32F_ARB = Alpha32F
+   | y == GL_INTENSITY32F_ARB = Intensity32F
+   | y == GL_LUMINANCE32F_ARB = Luminance32F
+   | y == GL_LUMINANCE_ALPHA32F_ARB = LuminanceAlpha32F
+   | y == GL_ALPHA16F_ARB = Alpha16F
+   | y == GL_INTENSITY16F_ARB = Intensity16F
+   | y == GL_LUMINANCE16F_ARB = Luminance16F
+   | y == GL_LUMINANCE_ALPHA16F_ARB = LuminanceAlpha16F
+   | y == GL_DEPTH24_STENCIL8_EXT = Depth24Stencil8
    -- legacy values
    | y == 1 = Luminance'
    | y == 2 = LuminanceAlpha'
